from flask import Flask, request, render_template, send_file, jsonify
from lxml import etree
import io
import datetime

app = Flask(__name__)

def is_wxr_format(xml_content):
    try:
        root = etree.fromstring(xml_content)
        # rss要素とchannel要素の確認
        if root.tag == 'rss' and root.find('channel') is not None:
            # WordPressの名前空間を確認
            wp_namespace = 'http://wordpress.org/export/1.2/'
            if root.find(f".//{{{wp_namespace}}}base_site_url") is not None:
                return True
        return False
    except etree.ParseError:
        return False

def convert_to_mt(xml_content):
    tree = etree.parse(io.BytesIO(xml_content))
    root = tree.getroot()
    channel = root.find('channel')
    
    # ベースURLの取得
    base_site_url = channel.find('{http://wordpress.org/export/1.2/}base_site_url')
    base_url = base_site_url.text if base_site_url is not None else ''
    
    output = []
    
    for item in channel.findall('item'):
        # post_typeの確認
        post_type = item.find('{http://wordpress.org/export/1.2/}post_type')
        if post_type is not None and post_type.text != 'post':
            continue  # post以外はスキップ
            
        title_elem = item.find('title')
        content_elem = item.find('{http://purl.org/rss/1.0/modules/content/}encoded')
        date_elem = item.find('{http://wordpress.org/export/1.2/}post_date')
        creator_elem = item.find('{http://purl.org/dc/elements/1.1/}creator')
        link_elem = item.find('link')
        
        # リンクURLからベースネームを抽出
        basename = ''
        if link_elem is not None and base_url:
            full_url = link_elem.text
            basename = full_url.replace(base_url, '').strip('/')
        
        # カテゴリー情報の取得
        categories = []
        primary_category = ""
        for category in item.findall('category'):
            if category.get('domain') == 'category':
                cat_name = category.text
                if not primary_category:
                    primary_category = cat_name
                else:
                    categories.append(cat_name)
        
        # 値の安全な取得
        title = title_elem.text if title_elem is not None else '無題'
        content = content_elem.text if content_elem is not None else ''
        post_date = date_elem.text if date_elem is not None else datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        author = creator_elem.text if creator_elem is not None else 'admin'
        
        # カテゴリー行の生成
        category_lines = []
        if primary_category:
            category_lines.append(f"PRIMARY CATEGORY: {primary_category}")
        if categories:
            category_lines.extend(f"CATEGORY: {cat}" for cat in categories)
        
        mt_format = f"""AUTHOR: {author}
TITLE: {title}
BASENAME: {basename}
STATUS: Publish
DATE: {post_date}
{chr(10).join(category_lines)}
-----
BODY:
{content}
-----
EXTENDED BODY:
-----
EXCERPT:
-----
KEYWORDS:
-----
--------
"""
        output.append(mt_format)
    
    return '\n'.join(output)

@app.route('/')
def index():
    return render_template('upload.html')

@app.route('/convert', methods=['POST'])
def convert():
    if 'file' not in request.files:
        return jsonify({'error': '変換するファイルが選択されていません'}), 400
    
    file = request.files['file']
    if file.filename == '':
        return jsonify({'error': 'ファイルが選択されていません'}), 400
        
    #if not file.filename.endswith('.xml'):
    #    return jsonify({'error': 'XMLファイルを選択してください'}), 400
    
    try:
        xml_content = file.read()
        
        if not is_wxr_format(xml_content):
            return jsonify({'error': 'WXR形式のファイルではありません'}), 400
            
        mt_content = convert_to_mt(xml_content)
        
        output = io.BytesIO(mt_content.encode('utf-8'))
        timestamp = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
        return send_file(
            output,
            mimetype='text/plain',
            as_attachment=True,
            download_name=f'mt_export_{timestamp}.txt'
        )
        
    except Exception as e:
        return jsonify({'error': f'変換エラーが発生しました: {str(e)}'}), 500

