#!/bin/bash
# Copyright 2025 zenhp.co.jp
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

# ログファイルの設定
LOG_FILE="setup_log.txt"
rm ${LOG_FILE}
exec 1> >(tee -a "$LOG_FILE")
exec 2>&1

# インストールするパッケージの定義
PACKAGES=(
    "Flask"
    "flup"
    "lxml"
)

echo "Flask環境構築を開始します..."
echo "実行日時: $(date)"

# 現在の日時を取得（バックアップ用）
DATETIME=$(date '+%Y%m%d_%H%M%S')

# condaコマンドの存在確認と環境のチェック
if [ -d "$HOME/miniconda3" ] && [ -f "$HOME/miniconda3/bin/conda" ]; then
    echo "Minicondaはすでにインストールされています"
    # PATHが設定されていない場合は設定
    if ! command -v conda >/dev/null 2>&1; then
        echo "環境変数を設定しています..."
        export PATH="$HOME/miniconda3/bin:$PATH"
        source ~/.bashrc
    fi
else
    echo "Minicondaをダウンロードしています..."
    wget -nv https://repo.anaconda.com/miniconda/Miniconda3-latest-Linux-x86_64.sh

    echo "Minicondaをインストールしています..."
    bash Miniconda3-latest-Linux-x86_64.sh -b -p $HOME/miniconda3

    echo "環境変数を設定しています..."
    echo 'export PATH="$HOME/miniconda3/bin:$PATH"' >> ~/.bashrc
    source ~/.bashrc

    echo "Condaを初期化しています..."
    $HOME/miniconda3/bin/conda init bash
    source ~/.bashrc
fi


# Python環境の確認と作成
if conda env list | grep -q "py310"; then
    echo "Python 3.10環境はすでに存在します"
else
    echo "Python 3.10環境を作成しています..."
    $HOME/miniconda3/bin/conda create --name py310 python=3.10 -y
fi

# 必要なパッケージのインストール
echo "パッケージの確認とインストールを行います..."
source $HOME/miniconda3/bin/activate py310

# パッケージのインストール処理
for package_name in "${PACKAGES[@]}"; do
    if python -c "import ${package_name,,}" 2>/dev/null; then
        echo "${package_name}はすでにインストールされています"
    else
        echo "${package_name}をインストールしています..."
        python -m pip install "$package_name" --progress-bar off
    fi
done

# FastCGI用ファイルのバックアップと作成
echo "FastCGI用ファイルを作成しています..."

# index.fcgiのバックアップと作成
if [ -f "index.fcgi" ]; then
    echo "既存のindex.fcgiをバックアップしています..."
    cp index.fcgi "index.fcgi.${DATETIME}"
fi

# index.fcgiの作成
cat > index.fcgi << EOL
#! ${HOME}/miniconda3/envs/py310/bin/python
from flup.server.fcgi import WSGIServer
from app import app

if __name__ == '__main__':
    WSGIServer(app).run()
EOL

# .htaccessのバックアップと作成
if [ -f ".htaccess" ]; then
    echo "既存の.htaccessをバックアップしています..."
    cp .htaccess ".htaccess.${DATETIME}"
fi

# .htaccessの作成
cat > .htaccess << 'EOL'
<IfModule mod_rewrite.c>
RewriteEngine On
RewriteCond %{REQUEST_FILENAME} !-f
RewriteRule ^(.*)$ /index.fcgi/$1 [L]
</IfModule>
EOL

# パーミッションの設定
chmod 755 index.fcgi

# Minicondaインストーラーの削除（存在する場合）
if [ -f "Miniconda3-latest-Linux-x86_64.sh" ]; then
    echo "Minicondaインストーラーを削除しています..."
    rm Miniconda3-latest-Linux-x86_64.sh
fi

echo "セットアップが完了しました"

